@@ -36,11 +36,6 @@ import org.apache.commons.cli.DefaultParser;
 import org.apache.commons.cli.HelpFormatter;
 import org.apache.commons.cli.Options;
 import org.apache.commons.cli.ParseException;
-import org.apache.gobblin.configuration.State;
-import org.apache.gobblin.instrumented.Instrumented;
-import org.apache.gobblin.instrumented.StandardMetricsBridge;
-import org.apache.gobblin.metrics.MetricContext;
-import org.apache.gobblin.metrics.Tag;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
@@ -63,9 +58,7 @@ import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Throwables;
-import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
-import com.google.common.collect.Maps;
 import com.google.common.eventbus.EventBus;
 import com.google.common.util.concurrent.MoreExecutors;
 import com.google.common.util.concurrent.Service;
@@ -74,20 +67,19 @@ import com.typesafe.config.Config;
 import com.typesafe.config.ConfigFactory;
 import com.typesafe.config.ConfigValueFactory;
 
+import javax.annotation.Nonnull;
+
 import org.apache.gobblin.annotation.Alpha;
 import org.apache.gobblin.configuration.ConfigurationKeys;
+import org.apache.gobblin.instrumented.StandardMetricsBridge;
 import org.apache.gobblin.metrics.GobblinMetrics;
-import org.apache.gobblin.runtime.TaskExecutor;
-import org.apache.gobblin.runtime.TaskStateTracker;
-import org.apache.gobblin.runtime.services.JMXReportingService;
+import org.apache.gobblin.metrics.MetricContext;
+import org.apache.gobblin.util.ClassAliasResolver;
 import org.apache.gobblin.util.ConfigUtils;
 import org.apache.gobblin.util.FileUtils;
 import org.apache.gobblin.util.HadoopUtils;
 import org.apache.gobblin.util.JvmUtils;
-import org.apache.gobblin.util.PathUtils;
-
-import javax.annotation.Nonnull;
-import lombok.Getter;
+import org.apache.gobblin.util.reflection.GobblinConstructorUtils;
 
 import static org.apache.gobblin.cluster.GobblinClusterConfigurationKeys.CLUSTER_WORK_DIR;
 
@@ -97,9 +89,10 @@ import static org.apache.gobblin.cluster.GobblinClusterConfigurationKeys.CLUSTER
  * {@link org.apache.gobblin.source.workunit.WorkUnit}s.
  *
  * <p>
- *   This class serves as a Helix participant and it uses a {@link HelixManager} to work with Helix.
- *   This class also uses the Helix task execution framework and {@link GobblinHelixTaskFactory} class
- *   for creating {@link GobblinHelixTask}s that Helix manages to run Gobblin data ingestion tasks.
+ *   This class presents a Helix participant and uses a {@link HelixManager} to communicate with Helix.
+ *   It also uses Helix task execution framework and {@link GobblinHelixTaskFactory} class to generate
+ *   {@link GobblinHelixTask}s which handles real Gobblin tasks. All the Helix related task framework is
+ *   encapsulated in {@link TaskRunnerSuiteBase}.
  * </p>
  *
  * <p>
@@ -124,8 +117,12 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
 
   static final String GOBBLIN_TASK_FACTORY_NAME = "GobblinTaskFactory";
 
+  static final String GOBBLIN_JOB_FACTORY_NAME = "GobblinJobFactory";
+
   private final String helixInstanceName;
 
+  private final String clusterName;
+
   private HelixManager helixManager;
 
   private final ServiceManager serviceManager;
@@ -149,6 +146,7 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
   private final String applicationName;
   private final String applicationId;
   private final Path appWorkPath;
+
   private final MetricContext metricContext;
   private final StandardMetricsBridge.StandardMetrics metrics;
 
@@ -162,21 +160,33 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
 
     Configuration conf = HadoopUtils.newConfiguration();
     this.fs = buildFileSystem(config, conf);
-
     this.appWorkPath = initAppWorkDir(config, appWorkDirOptional);
-
     this.config = saveConfigToFile(config);
+    this.clusterName = this.config.getString(GobblinClusterConfigurationKeys.HELIX_CLUSTER_NAME_KEY);
 
     initHelixManager();
 
     this.containerMetrics = buildContainerMetrics();
-    TaskFactoryBuilder builder = new TaskFactoryBuilder(this.config);
-    this.taskStateModelFactory = createTaskStateModelFactory(builder.build());
-    this.metrics = builder.getTaskMetrics();
-    this.metricContext = builder.getMetricContext();
 
-    services.addAll(getServices());
-    if (services.isEmpty()) {
+    String builderStr = ConfigUtils.getString(this.config, GobblinClusterConfigurationKeys.TASK_RUNNER_SUITE_BUILDER, TaskRunnerSuiteBase.Builder.class.getName());
+    TaskRunnerSuiteBase.Builder builder = GobblinConstructorUtils.<TaskRunnerSuiteBase.Builder>invokeLongestConstructor(
+          new ClassAliasResolver(TaskRunnerSuiteBase.Builder.class).resolveClass(builderStr), this.config);
+
+    TaskRunnerSuiteBase suite = builder.setAppWorkPath(this.appWorkPath)
+        .setContainerMetrics(this.containerMetrics)
+        .setFileSystem(this.fs)
+        .setHelixManager(this.helixManager)
+        .setApplicationId(applicationId)
+        .setApplicationName(applicationName)
+        .build();
+
+    this.taskStateModelFactory = createTaskStateModelFactory(suite.getTaskFactoryMap());
+    this.metrics = suite.getTaskMetrics();
+    this.metricContext = suite.getMetricContext();
+    this.services.addAll(suite.getServices());
+
+    this.services.addAll(getServices());
+    if (this.services.isEmpty()) {
       this.serviceManager = null;
     } else {
       this.serviceManager = new ServiceManager(services);
@@ -186,38 +196,6 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
         applicationName, helixInstanceName, applicationId, taskRunnerId, config, appWorkDirOptional);
   }
 
-  private class TaskFactoryBuilder {
-    private final boolean isRunTaskInSeparateProcessEnabled;
-    private final TaskFactory taskFactory;
-    @Getter
-    private final MetricContext metricContext;
-    @Getter
-    private StandardMetricsBridge.StandardMetrics taskMetrics;
-
-    public TaskFactoryBuilder(Config config) {
-      isRunTaskInSeparateProcessEnabled = getIsRunTaskInSeparateProcessEnabled(config);
-      metricContext = Instrumented.getMetricContext(ConfigUtils.configToState(config), this.getClass());
-      if (isRunTaskInSeparateProcessEnabled) {
-        logger.info("Running a task in a separate process is enabled.");
-        taskFactory = new HelixTaskFactory(GobblinTaskRunner.this.containerMetrics, CLUSTER_CONF_PATH, config);
-        taskMetrics = new GobblinTaskRunnerMetrics.JvmTaskRunnerMetrics();
-      } else {
-        Properties properties = ConfigUtils.configToProperties(config);
-        TaskExecutor taskExecutor = new TaskExecutor(properties);
-        taskFactory = getInProcessTaskFactory(taskExecutor);
-        taskMetrics = new GobblinTaskRunnerMetrics.InProcessTaskRunnerMetrics(taskExecutor, metricContext);
-      }
-    }
-
-    public TaskFactory build(){
-       return taskFactory;
-    }
-
-    private Boolean getIsRunTaskInSeparateProcessEnabled(Config config) {
-      return ConfigUtils.getBoolean(config, GobblinClusterConfigurationKeys.ENABLE_TASK_IN_SEPARATE_PROCESS, false);
-    }
-  }
-
   private Path initAppWorkDir(Config config, Optional<Path> appWorkDirOptional) {
     return appWorkDirOptional.isPresent() ? appWorkDirOptional.get() : GobblinClusterUtils
         .getAppWorkDirPathFromConfig(config, this.fs, this.applicationName, this.applicationId);
@@ -229,14 +207,10 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
     logger.info("Using ZooKeeper connection string: " + zkConnectionString);
 
     this.helixManager = HelixManagerFactory.getZKHelixManager(
-        this.config.getString(GobblinClusterConfigurationKeys.HELIX_CLUSTER_NAME_KEY),
-        this.helixInstanceName, InstanceType.PARTICIPANT, zkConnectionString);
+        this.clusterName, this.helixInstanceName, InstanceType.PARTICIPANT, zkConnectionString);
   }
 
-  private TaskStateModelFactory createTaskStateModelFactory(TaskFactory factory) {
-    Map<String, TaskFactory> taskFactoryMap = Maps.newHashMap();
-
-    taskFactoryMap.put(GOBBLIN_TASK_FACTORY_NAME, factory);
+  private TaskStateModelFactory createTaskStateModelFactory(Map<String, TaskFactory> taskFactoryMap) {
     TaskStateModelFactory taskStateModelFactory =
         new TaskStateModelFactory(this.helixManager, taskFactoryMap);
     this.helixManager.getStateMachineEngine()
@@ -244,35 +218,6 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
     return taskStateModelFactory;
   }
 
-  private TaskFactory getInProcessTaskFactory(TaskExecutor taskExecutor) {
-    Properties properties = ConfigUtils.configToProperties(this.config);
-    URI rootPathUri = PathUtils.getRootPath(this.appWorkPath).toUri();
-    Config stateStoreJobConfig = ConfigUtils.propertiesToConfig(properties)
-        .withValue(ConfigurationKeys.STATE_STORE_FS_URI_KEY,
-            ConfigValueFactory.fromAnyRef(rootPathUri.toString()));
-
-    TaskStateTracker taskStateTracker = new GobblinHelixTaskStateTracker(properties);
-
-    services.add(taskExecutor);
-    services.add(taskStateTracker);
-    services.add(new JMXReportingService(
-        ImmutableMap.of("task.executor", taskExecutor.getTaskExecutorQueueMetricSet())));
-
-    TaskFactory taskFactory =
-        new GobblinHelixTaskFactory(this.containerMetrics, taskExecutor, taskStateTracker, this.fs,
-            this.appWorkPath, stateStoreJobConfig, this.helixManager);
-    return taskFactory;
-  }
-
-  private Boolean getIsRunTaskInSeparateProcessEnabled() {
-    Boolean enabled = false;
-    if (this.config.hasPath(GobblinClusterConfigurationKeys.ENABLE_TASK_IN_SEPARATE_PROCESS)) {
-      enabled =
-          this.config.getBoolean(GobblinClusterConfigurationKeys.ENABLE_TASK_IN_SEPARATE_PROCESS);
-    }
-    return enabled;
-  }
-
   private Config saveConfigToFile(Config config)
       throws IOException {
     Config newConf = config
@@ -294,6 +239,8 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
 
     connectHelixManager();
 
+    addInstanceTags();
+
     // Start metric reporting
     if (this.containerMetrics.isPresent()) {
       this.containerMetrics.get()
@@ -374,6 +321,18 @@ public class GobblinTaskRunner implements StandardMetricsBridge {
     }
   }
 
+  /**
+   * Helix participant cannot pre-configure tags before it connects to ZK. So this method can only be invoked after
+   * {@link HelixManager#connect()}. However this will still work because tagged jobs won't be sent to a non-tagged instance. Hence
+   * the job with EXAMPLE_INSTANCE_TAG will remain in the ZK until an instance with EXAMPLE_INSTANCE_TAG was found.
+   */
+  private void addInstanceTags() {
+    if (this.helixManager.isConnected()) {
+      List<String> tags = ConfigUtils.getStringList(this.config, GobblinClusterConfigurationKeys.HELIX_INSTANCE_TAGS_KEY);
+      tags.forEach(tag -> helixManager.getClusterManagmentTool().addInstanceTag(this.clusterName, this.helixInstanceName, tag));
+    }
+  }
+
   /**
    * Creates and returns a {@link MessageHandlerFactory} for handling of Helix
    * {@link org.apache.helix.model.Message.MessageType#USER_DEFINE_MSG}s.
