@@ -31,6 +31,13 @@ import javax.annotation.Nonnull;
 import lombok.extern.slf4j.Slf4j;
 
 import org.apache.commons.lang.StringUtils;
+import org.apache.gobblin.configuration.SourceState;
+import org.apache.gobblin.data.management.conversion.hive.dataset.ConvertibleHiveDataset;
+import org.apache.gobblin.data.management.conversion.hive.source.HiveAvroToOrcSource;
+import org.apache.gobblin.data.management.conversion.hive.utils.LineageUtils;
+import org.apache.gobblin.dataset.DatasetDescriptor;
+import org.apache.gobblin.metrics.event.lineage.LineageInfo;
+import org.apache.gobblin.source.workunit.WorkUnit;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
@@ -90,6 +97,7 @@ public class HiveConvertPublisher extends DataPublisher {
   private final FileSystem fs;
   private final HiveSourceWatermarker watermarker;
   private final HiveMetastoreClientPool pool;
+  private final Optional<LineageInfo> lineageInfo;
 
   public static final String PARTITION_PARAMETERS_WHITELIST = "hive.conversion.partitionParameters.whitelist";
   public static final String PARTITION_PARAMETERS_BLACKLIST = "hive.conversion.partitionParameters.blacklist";
@@ -105,6 +113,15 @@ public class HiveConvertPublisher extends DataPublisher {
     this.metricContext = Instrumented.getMetricContext(state, HiveConvertPublisher.class);
     this.eventSubmitter = new EventSubmitter.Builder(this.metricContext, EventConstants.CONVERSION_NAMESPACE).build();
 
+    // Extract LineageInfo from state
+    if (state instanceof SourceState) {
+      lineageInfo = LineageInfo.getLineageInfo(((SourceState) state).getBroker());
+    } else if (state instanceof WorkUnitState) {
+      lineageInfo = LineageInfo.getLineageInfo(((WorkUnitState) state).getTaskBrokerNullable());
+    } else {
+      lineageInfo = Optional.absent();
+    }
+
     Configuration conf = new Configuration();
     Optional<String> uri = Optional.fromNullable(this.state.getProp(ConfigurationKeys.WRITER_FILE_SYSTEM_URI));
     if (uri.isPresent()) {
@@ -226,6 +243,9 @@ public class HiveConvertPublisher extends DataPublisher {
             } catch (Exception e) {
               log.error("Failed while emitting SLA event, but ignoring and moving forward to curate " + "all clean up commands", e);
             }
+            if (LineageUtils.shouldSetLineageInfo(wus)) {
+              setDestLineageInfo(wus, this.lineageInfo);
+            }
           }
         }
       }
@@ -253,6 +273,18 @@ public class HiveConvertPublisher extends DataPublisher {
     }
   }
 
+  @VisibleForTesting
+  public static void setDestLineageInfo(WorkUnitState wus, Optional<LineageInfo> lineageInfo) {
+    HiveWorkUnit hiveWorkUnit = new HiveWorkUnit(wus.getWorkunit());
+    ConvertibleHiveDataset convertibleHiveDataset = (ConvertibleHiveDataset) hiveWorkUnit.getHiveDataset();
+    List<DatasetDescriptor> destDatasets = convertibleHiveDataset.getDestDatasets();
+    for (int i = 0; i < destDatasets.size(); i++) {
+      if (lineageInfo.isPresent()) {
+        lineageInfo.get().putDestination(destDatasets.get(i), i + 1, wus);
+      }
+    }
+  }
+
   @VisibleForTesting
   public void preservePartitionParams(Collection<? extends WorkUnitState> states) {
     for (WorkUnitState wus : states) {
