@@ -27,6 +27,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 
+import org.apache.gobblin.metastore.DatasetStateStore;
 import org.apache.hadoop.io.Text;
 
 import com.codahale.metrics.Counter;
@@ -53,6 +54,7 @@ import org.apache.gobblin.rest.Metric;
 import org.apache.gobblin.rest.MetricArray;
 import org.apache.gobblin.rest.MetricTypeEnum;
 import org.apache.gobblin.rest.TaskExecutionInfoArray;
+import org.apache.gobblin.runtime.api.MonitoredObject;
 import org.apache.gobblin.runtime.util.JobMetrics;
 import org.apache.gobblin.runtime.util.MetricGroup;
 import org.apache.gobblin.source.extractor.JobCommitPolicy;
@@ -84,7 +86,7 @@ public class JobState extends SourceState {
    *    <li> SUCCESSFUL => CANCELLED  (cancelled before committing)
    * </ul>
    */
-  public enum RunningState {
+  public enum RunningState implements MonitoredObject {
     /** Pending creation of {@link WorkUnit}s. */
     PENDING,
     /** Starting the execution of {@link WorkUnit}s. */
@@ -131,6 +133,7 @@ public class JobState extends SourceState {
   private final Map<String, TaskState> taskStates = Maps.newLinkedHashMap();
   // Skipped task states shouldn't be exposed to publisher, but they need to be in JobState and DatasetState so that they can be written to StateStore.
   private final Map<String, TaskState> skippedTaskStates = Maps.newLinkedHashMap();
+  private DatasetStateStore datasetStateStore;
 
   // Necessary for serialization/deserialization
   public JobState() {
@@ -142,6 +145,13 @@ public class JobState extends SourceState {
     this.setId(jobId);
   }
 
+  public JobState(State properties,String jobName, String jobId) {
+    super(properties);
+    this.jobName = jobName;
+    this.jobId = jobId;
+    this.setId(jobId);
+  }
+
   public JobState(State properties, Map<String, JobState.DatasetState> previousDatasetStates, String jobName,
       String jobId) {
     super(properties, previousDatasetStates, workUnitStatesFromDatasetStates(previousDatasetStates.values()));
@@ -728,7 +738,7 @@ public class JobState extends SourceState {
     return datasetState;
   }
 
-  private static List<WorkUnitState> workUnitStatesFromDatasetStates(Iterable<JobState.DatasetState> datasetStates) {
+  public static List<WorkUnitState> workUnitStatesFromDatasetStates(Iterable<JobState.DatasetState> datasetStates) {
     ImmutableList.Builder<WorkUnitState> taskStateBuilder = ImmutableList.builder();
     for (JobState datasetState : datasetStates) {
       taskStateBuilder.addAll(datasetState.getTaskStatesAsWorkUnitStates());
